//
//  ****************************************************************************
//
//  bond.h
//
//  (C) Thu Mar 25 2004 Bosco K. Ho 
//
//  ****************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  ****************************************************************************
//
//

#ifndef _bond_h
#define _bond_h

#include <list>
#include "protein.h"



class BondStruct
{
  public:
    BondStruct(int i, int c, int o, int j, int n, int h) :
      acceptorRes(i), neighbour(c), acceptor(o), donorRes(j), donor(n), hydrogen(h) {}
    int acceptorRes, neighbour, acceptor, donorRes, donor, hydrogen;
};



class HBondList
{
  public:
    HBondList(void) {}
    void checkBond(Protein* pProtein, int i, int c, int o, int j, int n, int h);
    void countDonors(Protein* pProtein, int res1No, int c, int o, int res2No);
    void countBetweenResidues(Protein* pProtein, int res1No, int res2No);
    void countBetweenBackbones(Protein* pProtein, int res1No, int res2No);
    void countWithinResidue(Protein* pProtein, int i);
    void clear(void) { list.clear(); };
    std::deque<BondStruct> list;
};



class WeakHBondList: public HBondList
{
  public:
    void countBond(Protein* pProtein, int i, int c, int o, int j, int n, int h);
    void countDonors(Protein* pProtein, int res1No, int c, int o, int res2No);
    void countBetweenResidues(Protein* pProtein, int res1No, int res2No);
    void countBetweenBackbones(Protein* pProtein, int res1No, int res2No);
};



class DisulfideList: public HBondList
{
  public:
    void countBetweenResidues(Protein* pProtein, int i, int j);
};


#endif
