//
//  ***************************************************************************
//
//  control.cpp
//  (C) 2004 Bosco K. Ho 
//
//  Various decorator widgets for ProteinDisplay that manipulate
//  the pProtein embedded in ProteinDisplay.
//
//  ***************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  ****************************************************************************
//


#include "control.h"
#include <iomanip>
#include <fstream>

using namespace std;

/// Ramachandran plot



int RamaBox::handle(int event)
{
  if ((event == WGT_MOUSE_B1_DOWN) || (event == WGT_MOUSE_B1_DRAG))
  {
    double phi = ((mouse_x - mX - 2) / ((double)mW - 4) * 360.0) - 180.0;

    double psi = -(((mouse_y - mY - 2) / ((double)mH - 4) * 360.0) - 180.0);

    normalize_angle(phi);
    normalize_angle(psi);

    if (mpDisplay->anchorRes3 != -1)
      mpDisplay->changeAnchorPhiPsi(phi, psi);
    else
      for (int i=mpDisplay->activeLeftRes; i<=mpDisplay->activeRightRes; i+=1)
      {
        if (mpDisplay->pProtein->resKey(i) == PRO)
          mpDisplay->pProtein->rotateProlinePhi(i, phi);
        else
          mpDisplay->pProtein->rotatePhi(i, phi);
        mpDisplay->pProtein->rotatePsi(i, psi);
      }

    mpDisplay->resetFocusWithSameOrientation();
    return 1;
  }

  return 0;
}



void RamaBox::draw(BITMAP *bmp)
{
  if (mpDisplay->pProtein->isEmpty())
    return;
    
  text_mode(-1);
 
  rect(bmp, mX, mY, mX + mW - 1, mY + mW - 1, GREEN);

  line(bmp, mX + (mW-1)/2, mY + 1, mX + (mW-1)/2, mY + mH - 2, DARK_GREY);
  line(bmp, mX + 1, mY + (mH-1)/2, mX + mW - 2, mY + (mH-1)/2, DARK_GREY);

  int i;
  for (i=mpDisplay->activeLeftRes; i<=mpDisplay->activeRightRes; i+=1)
  {
    if ( (mpDisplay->pProtein->resKey(i) == ACE) || (mpDisplay->pProtein->resKey(i) == NME) )
      continue; 
    int rX = (int)(( mpDisplay->pProtein->phi(i)+180) / 360.0 * (mW-4) + mX  + 2);
    int rY = (int)((-mpDisplay->pProtein->psi(i)+180) / 360.0 * (mH-4) + mY  + 2);

    line(bmp, rX, mY + 1, rX, mY + mH - 2, DARK_GREEN);
    line(bmp, mX + 1, rY, mX + mW - 2, rY, DARK_GREEN);
  }

  for (i=mpDisplay->activeLeftRes; i<=mpDisplay->activeRightRes; i+=1)
  {
    stringout(bmp, font, getResName3(mpDisplay->pProtein->resKey(i)) + ":"
       + double_to_string(mpDisplay->pProtein->resNo(i), 0, 0), mX + 5, mY + 5, GREEN);

    if ( (mpDisplay->pProtein->resKey(i) == ACE) || (mpDisplay->pProtein->resKey(i) == NME) )
      continue; 
    float phi = (float) mpDisplay->pProtein->phi(i);
    float psi = (float) mpDisplay->pProtein->psi(i);

    int rX = (int)(( phi+180) / 360.0 * (mW-4) + mX + 2);
    int rY = (int)((-psi+180) / 360.0 * (mH-4) + mY + 2);

    stringout(bmp, font, "phi " + double_to_string(phi, 4, 0), mX + mW - 65, mY +  5, GREEN);
    stringout(bmp, font, "psi " + double_to_string(psi, 4, 0), mX + mW - 65, mY + 17, GREEN);

    circlefill(bmp, rX, rY, 4, BLACK);
    if ( (i==mpDisplay->anchorRes1) || (i==mpDisplay->anchorRes2)
      || (i==mpDisplay->anchorRes3) )
    {
      circlefill(bmp, rX, rY, 3, BLUE);
      circle(bmp, rX, rY, 5, BLUE);
    }
    else
      circlefill(bmp, rX, rY, 3, GREEN);
  }
}



/// Chi angle control box



bool ChiBox::isInside(int x, int y)
{
  if (mpDisplay->pProtein->isEmpty()) return false;
  if (mpDisplay->activeLeftRes != mpDisplay->activeRightRes) return false;
  if (getNChi(mpDisplay->pProtein->resKey(mpDisplay->activeLeftRes)) < 1) return false;

  int actual_height;
  if (mpDisplay->pProtein->resKey(mpDisplay->activeLeftRes) == PRO)
    actual_height = 5*mH;
  else  
    actual_height = getNChi(mpDisplay->pProtein->resKey(mpDisplay->activeLeftRes))*mH;
  return((mouse_x >= mX) && (mouse_x <(mX + mW))
        && (mouse_y >= mY + 4*mH - actual_height)
        && (mouse_y <= mY + 4*mH) );
}



int ChiBox::handle(int event)
{
  switch (event)
  {
    case WGT_MOUSE_B1_DOWN:
      iChi = -1;
    case WGT_MOUSE_B1_DRAG:
    {
       if (mpDisplay->activeLeftRes != mpDisplay->activeRightRes) return 0;

      if (iChi == -1)
        iChi = (int)( 4 - (mouse_y - mY)/(double)mH );

      if ( (mpDisplay->pProtein->resKey(mpDisplay->activeLeftRes) == PRO) )
      {
        if (iChi == 4) 
        {
          int& i = mpDisplay->activeLeftRes;
          if (mouse_x < mX + mW/2)
            mpDisplay->pProtein->proPucker(i) = -1.0;
          else
            mpDisplay->pProtein->proPucker(i) = 1.0;
          mpDisplay->pProtein->rotateProlinePhi(i, mpDisplay->pProtein->phi(i));
        }
      }
      else if ((iChi >= 0) && (iChi < getNChi(mpDisplay->pProtein->resKey(mpDisplay->activeLeftRes))))
      {
        double chi = ( 360.0 * (mouse_x - mX - 2) / (double)(mW - 4) + -180.0 );
        normalize_angle(chi);
        mpDisplay->pProtein->rotateChi(mpDisplay->activeLeftRes, iChi, chi);
        mpDisplay->resetFocusWithSameOrientation();
      }
      
    }
    break;
    default:
      return 0;
  }
  return 1;
}



void ChiBox::draw(BITMAP *bmp)
{
  if (mpDisplay->pProtein->isEmpty())
    return;    
  int i = mpDisplay->activeLeftRes;
  if (i != mpDisplay->activeRightRes) return;
  if (getNChi(mpDisplay->pProtein->resKey(i)) < 1) return;

  text_mode(-1);

  for (int j = getNChi(mpDisplay->pProtein->resKey(i)) - 1; j >= 0; j--)
  {
    int fore;
    if ( (mpDisplay->pProtein->resKey(i) == PRO) )
      fore = GREY;
    else
      fore = GREEN;
    float chi = mpDisplay->pProtein->chi(i, j);
    int rX = (int)( (mW - 4) * (chi - (-180.0)) / 360.0 + mX + 2 );
    int rY = mY + mH * (4 - j - 1);
    stringout(bmp, font, "chi" + double_to_string(j+1, 0, 0)
      + double_to_string(chi, 4, 0), mX + mW - 65, rY + 5, GREEN);
    rect(bmp, mX, rY, mX + mW - 1, rY + mH, fore);
    circlefill(bmp, rX, rY + mH/2, 4, BLACK);
    circlefill(bmp, rX, rY + mH/2, 3, fore);
  }

  if ( (mpDisplay->pProtein->resKey(i) == PRO) )
  {
    int rY = mY + mH * (4 - 4 - 1);
    rect(bmp, mX, rY, mX + mW - 1, rY + mH, GREEN);
    if (mpDisplay->pProtein->proPucker(i) > 0.0) 
    {
      stringout_centre (bmp, font, "DOWN", mX + mW/4,      rY + 4,                GREEN);
      rectfill         (bmp,               mX + mW/2,      rY, mX + mW, rY + mH,  GREEN);
      stringout_centre (bmp, font, "UP",   mX + 0.75 * mW, rY + 4,                BLACK);
    }
    else
    {
      rectfill         (bmp,               mX,             rY, mX  + 0.5 * mW, rY + mH, GREEN);
      stringout_centre (bmp, font, "DOWN", mX + 0.25 * mW, rY + 4,                      BLACK);
      stringout_centre (bmp, font, "UP",   mX + 0.75 * mW, rY + 4,                      GREEN);
    }
  }
}



/// Sequence Box control box

bool SequenceBox::isInside(int x, int y)

{
  return ( (x >= mX) && (x < (mX + (mpDisplay->pProtein->nRes()+1) * mW)) && (y >= mY) && (y <(mY + mH)) );
}



void SequenceBox::pasteClipboard()
{
  Protein& protein        = *(mpDisplay->pProtein);
  int&     activeLeftRes  = mpDisplay->activeLeftRes;
  int&     activeRightRes = mpDisplay->activeRightRes;
  int&     viewLeftRes    = mpDisplay->viewLeftRes;
  int&     viewRightRes   = mpDisplay->viewRightRes;
  int&     focusRes       = mpDisplay->focusRes;

  if (mClipboardProtein.isEmpty()) 
    return;

  if (protein.isEmpty()) 
  {
    activeLeftRes = 0;
    focusRes = 0;
  }

  if (ACE == protein.resKey(activeLeftRes))
    activeLeftRes++;

  bool isChargedNTerminal = false;
  bool isChargedCTerminal = false;
  
  if (0 == activeLeftRes)
  {
    isChargedNTerminal = protein.isChargedNTerminal();
    protein.neutralizeNTerminal();
  }
  
  if (protein.nRes() == activeLeftRes)
  {
    isChargedCTerminal = protein.isChargedCTerminal();
    protein.neutralizeCTerminal();
  }
    
  if (focusRes >= activeLeftRes)
    focusRes += mClipboardProtein.nRes();

  int size = mpDisplay->pProtein->nRes();
  if (mpDisplay->activeLeftRes == size)
  {
    if (!mpDisplay->pProtein->isEmpty())
    {
      if (NME == mpDisplay->pProtein->resKey(size - 1))
         mpDisplay->activeLeftRes = size - 1;
    }
    else
      activeLeftRes = 0;
  }

  protein.insertFragmentAndSplice(mClipboardProtein, activeLeftRes);
  activeRightRes = activeLeftRes + mClipboardProtein.nRes() - 1;
  
  if (isChargedNTerminal)
    protein.chargeNTerminal();

  if (isChargedCTerminal)
    protein.chargeCTerminal();

  checkBounds();
  mpDisplay->resetFocusWithSameOrientation();
}



void SequenceBox::deleteActiveRes()
{
  Protein& protein        = *(mpDisplay->pProtein);
  int&     activeLeftRes  = mpDisplay->activeLeftRes;
  int&     activeRightRes = mpDisplay->activeRightRes;
  int&     focusRes       = mpDisplay->focusRes;
  
  if ( (focusRes >= activeLeftRes) && (focusRes <= activeRightRes) )
    focusRes = activeLeftRes;
  else if (focusRes > activeRightRes)
    focusRes -= activeRightRes - activeLeftRes + 1;
  
  protein.eraseResAndSplice(activeLeftRes, activeRightRes);

  // make sure there are no proteins with just blocking groups
  if (protein.nRes() == 1)
    if ( (ACE == protein.resKey(0)) || (NME == protein.resKey(0)) )
    {
      protein.eraseRes(0);
    }

  if (protein.nRes() == 2)
    if ( (ACE == protein.resKey(0)) && (NME == protein.resKey(1)) )
    {
      protein.eraseRes(0);
      protein.eraseRes(1);
    }

  activeRightRes = activeLeftRes;
}



void SequenceBox::copyToClipboard()
{
  Protein& protein        = *(mpDisplay->pProtein);
  int&     activeLeftRes  = mpDisplay->activeLeftRes;
  int&     activeRightRes = mpDisplay->activeRightRes;

  mClipboardProtein.clear();
  protein.extractFragment(mClipboardProtein, activeLeftRes, activeRightRes);
  mClipboardProtein.neutralizeNTerminal();
  mClipboardProtein.neutralizeCTerminal();
  mClipboardProtein.unblock();
}



void SequenceBox::loadClipboardFromFile()
{
  // insert from files
  static char save_string[600] = "";
  if (file_select_ex("Insert pdb file", save_string, "pdb", 500, 600, 200) != 0)
  {
    PdbReader pdb(save_string);
    if (pdb.isFileLoaded()) 
    {
      pdb.loadChain(mClipboardProtein);
      mClipboardProtein.unblock();
      mClipboardProtein.neutralizeCTerminal();
      mClipboardProtein.neutralizeNTerminal();
    }
  }
}



int SequenceBox::handle(int event)
{
  Protein& protein        = *(mpDisplay->pProtein);
  int&     activeLeftRes  = mpDisplay->activeLeftRes;
  int&     activeRightRes = mpDisplay->activeRightRes;
  int&     viewLeftRes    = mpDisplay->viewLeftRes;
  int&     viewRightRes   = mpDisplay->viewRightRes;
  int&     focusRes       = mpDisplay->focusRes;
  
  switch (event)
  {
    case WGT_MOUSE_B1_DOWN:
    {
      int n = convertToSeqNo(mouse_x);

      if ((n >= 0) && (n < protein.nRes()))
      {
        activeLeftRes   = n;
        activeRightRes  = n;
        mAnchorRes  = n;
      }
    }
    break;

    case WGT_MOUSE_B1_DRAG:
    {
      int n;
      n = convertToSeqNo(mouse_x);
      if ((n >= 0) && (n < protein.nRes()))
      {
        if (n >= mAnchorRes)
        {
          activeRightRes = n;
          activeLeftRes  = mAnchorRes;
        }
        else
        {
          activeLeftRes  = n;
          activeRightRes = mAnchorRes;
        }
        
        if (SCROLLBAR_SIZE < protein.nRes() - 1)
        {
          if ( (n >= viewRightRes) && (n < protein.nRes()) )
          {
             viewRightRes = n;
             viewLeftRes = n - SCROLLBAR_SIZE + 1;
          }
          if ( (n <= viewLeftRes) && (n >= 0 ) )
          {
             viewLeftRes = n;
             viewRightRes = n + SCROLLBAR_SIZE - 1;
          }
        }
      }
    }
    return 1;
    break;

    case WGT_MOUSE_B2_DOWN:
    {
      int  n = convertToSeqNo(mouse_x);
      mIsMenu   = true;
      mMenuRes  = n;
      if ( ( (n >= 0) && (n < activeLeftRes) ) 
        || ( (n > activeRightRes) && (n < protein.nRes()) ) )
        {
          activeLeftRes  = n;
          activeRightRes = n;
        }
    }
    break;

    case WGT_MOUSE_B2_RELEASE:
    {
      if (!mIsMenu) 
        return 0;

      mIsMenu = false;

      int n = getMenuIndex(mouse_x, mouse_y);
 
      if (mMenuRes == protein.nRes())
      {
        if ((n >= 6) && (n <= 10))
        {
          loadClipboardFromFile();
        }
        else if ((n >= 11) && (n <= 15))
        {
          mClipboardProtein.clear();
          mClipboardProtein.appendRes(ALA);
          mClipboardProtein.appendRes(ALA);
          mClipboardProtein.appendRes(ALA);
          mClipboardProtein.appendRes(ALA);
        }
        else if ( !((n >= 1) && (n <= 5)) )
        {
          return 1;
        }
        if (mClipboardProtein.isEmpty())
          return 1;
        activeLeftRes = protein.nRes();
        pasteClipboard();
        viewRightRes = protein.nRes() - 1;
        viewLeftRes = viewRightRes - SCROLLBAR_SIZE + 1;
        checkBounds();
        mpDisplay->resetFocusWithSameOrientation();
      }
      else if ((activeLeftRes == activeRightRes) 
          && ( (protein.resKey(activeLeftRes) == ACE) || (protein.resKey(activeRightRes) == NME) ) )
      {
        if ((n >= 1) && (n <= 5))
        {
           pasteClipboard();
        }
        else if ((n >= 6) && (n <= 10))
        {
          deleteActiveRes();
        }
        else if ((n >= 11) && (n <= 15))
        {
          loadClipboardFromFile();
          if (mClipboardProtein.isEmpty()) 
            return 1;
          pasteClipboard();
        }
        checkBounds();
        mpDisplay->resetFocusWithSameOrientation();
      }
      else
      {
        if ((n >= 1) && (n <= 20))
        {
          for (int i = activeLeftRes; i <= activeRightRes; i++)
          {
            // mutate active residues
            protein.mutateRes(i, n);
            protein.sproutWholeSidechain(i);
          }
          return 1;
        }
        else if ((n >= 21) && (n <= 25))
        {
          copyToClipboard();
          return 1;
        }
        else if ((n >= 26) && (n <= 30))
        {
          copyToClipboard();
          deleteActiveRes();
        }
        else if ((n >= 31) && (n <= 35))
        {
          pasteClipboard();
        }
        else if ((n >= 36) && (n <= 40))
        {
          deleteActiveRes();
        }
        else if ((n >= 41) && (n <= 45))
        {
          loadClipboardFromFile();
          if (mClipboardProtein.isEmpty()) 
            return 1;
          pasteClipboard();
        }
        else if ((n >= 46) && (n <= 50))
        {
          // write to file
          static char save_string[600] = "";
          if (save_string[0] == 0) strcpy(save_string, "peptide.pdb");
      
          if (file_select_ex("Save pdb file", save_string, "pdb", 500, 600, 200) != 0)
          {
            std::ofstream out(save_string);
            if (!out.fail()) 
            {
              // copy residues to clipboard
              Protein frag;
              protein.extractFragment(frag, activeLeftRes, activeRightRes);
              // this is done so that we don't lose phi/psi info if 
              // no hydrogens are saved
              frag.blockNTerminal();
              frag.blockCTerminal();
            
              PdbWriter writer(save_string);
              writer.writeChain(frag, mpDisplay->isHydrogen);
            }
          }
          return 1;
        }
        else if ((n >= 51) && (n <= 55))
        // block end residues
        {
          if ( activeLeftRes == activeRightRes )
          {
            if ( (activeLeftRes == protein.nRes()-1) && (protein.resKey(protein.nRes() - 1) != NME) )
            {
              protein.blockCTerminal();
              viewLeftRes++;
              viewRightRes++;
            }
            if ( (activeLeftRes == 0) && (protein.resKey(0) != ACE) )
            {
              protein.blockNTerminal();
              activeLeftRes++;
              activeRightRes++;
              focusRes++;
            }
          }
        }
        else if ((n >= 56) && (n <= 60))
        // charge end residues
        {
          if ( activeLeftRes == activeRightRes )
          {
            if ( (activeLeftRes == protein.nRes()-1) && (protein.resKey(protein.nRes() - 1) != NME) )
            {
              protein.chargeCTerminal();
            }
            if ( (activeLeftRes == 0) && (protein.resKey(0) != ACE) )
            {
              protein.chargeNTerminal();
            }
          }
        }
        else if ((n >= 61) && (n <= 65))
        // neutralize end residues
        {
          if ( activeLeftRes == activeRightRes )
          {
            if ( (activeLeftRes == protein.nRes()-1) && (protein.resKey(protein.nRes() - 1) != NME) )
            {
              protein.neutralizeCTerminal();
            }
            if ( (activeLeftRes == 0) && (protein.resKey(0) != ACE) )
            {
              protein.neutralizeNTerminal();
            }
          }
        }
        checkBounds();
        mpDisplay->resetFocusWithSameOrientation();
        return 1;
      }
    }
    break;
  }
  return 0;
}


void SequenceBox::checkBounds()
{
  int&     viewLeftRes    = mpDisplay->viewLeftRes;
  int&     viewRightRes   = mpDisplay->viewRightRes;
  int&     activeLeftRes  = mpDisplay->activeLeftRes;
  int&     activeRightRes = mpDisplay->activeRightRes;
  int&     focusRes       = mpDisplay->focusRes;
  int&     focusAtom      = mpDisplay->focusAtom;
  Protein& protein        = *(mpDisplay->pProtein);
  int      lastRes        = protein.nRes() - 1;

  if ( protein.nRes() <= SCROLLBAR_SIZE )
  {
    viewLeftRes = 0;
    viewRightRes = lastRes;
  }

  while (viewRightRes > lastRes)
    viewRightRes--;

  if (SCROLLBAR_SIZE < protein.nRes()) 
  {
    int viewSize = viewRightRes - viewLeftRes + 1;
    while ((viewSize < SCROLLBAR_SIZE) && (viewRightRes < lastRes)) 
    {
      viewRightRes++;
      viewSize++;
    }
    while ((viewSize < SCROLLBAR_SIZE) && (viewLeftRes > 0)) 
    {
      viewLeftRes--;
      viewSize++;
    }
  }
  
  if (activeRightRes > lastRes)
  {
    activeRightRes = lastRes;
  }

  if (activeLeftRes > lastRes)
  {
    activeLeftRes = lastRes;
    activeRightRes = activeLeftRes;
  }
  
  if (focusRes > lastRes)
  {
    focusRes = lastRes;
    focusAtom = CA;
  }
}



int SequenceBox::convertToSeqNo(int x)
{
  return (int)((x - mX) /(double)mW + mpDisplay->viewLeftRes);
}



int SequenceBox::getMenuIndex(int x, int y)
{
  int tempX = (mMenuRes - mpDisplay->viewLeftRes) * mW + mX;
  int tempY = mY;

  int i = (int)((x - tempX) /(double)mW + 1 );
  if ( (i<1) || (i>5) ) return -1;
  
  int j = (int)((y - tempY) /(double)mH );

  return (j-1) * 5 + i;
}



void SequenceBox::drawMenuBlock(BITMAP *bmp, int n, int width, string s, int foreColor, int backColor)
{
  // reverse the equation n = (j-1)*5 + i;
  int j = (n-1) / 5 + 1;
  int i = n - (j-1) * 5;
  
  int tempX = (mMenuRes - mpDisplay->viewLeftRes) * mW + mX;
  int tempY = mY;

  int rX = tempX + (i-1) * mW;
  int rY = tempY + j * mH;

  rectfill (bmp, rX, rY, rX + mW * width, rY + mH, backColor);
  stringout_centre (bmp, font, s, rX + 0.5 * mW * width, rY + 3, foreColor);
  rect (bmp, rX, rY, rX + mW * width, rY + mH, WHITE);
}



void SequenceBox::drawInactiveMenuItem(BITMAP *bmp, int n, int width, string s)
{
  drawMenuBlock(bmp, n, width, s, GREY, BLACK); 
}



void SequenceBox::drawMenuItem(BITMAP *bmp, int n, int width, string s)
{
  int mouse_n = getMenuIndex(mouse_x, mouse_y);
  if ( (mouse_n >= n) && (mouse_n <= (n + width - 1)) )
    drawMenuBlock(bmp, n, width, s, BLACK, GREEN); 
  else
    drawMenuBlock(bmp, n, width, s, WHITE, BLACK); 
}



void SequenceBox::drawMenu(BITMAP *bmp)
{
  // menu for the end marker
  if (mMenuRes == mpDisplay->pProtein->nRes())
  {
    if (mClipboardProtein.isEmpty()) 
      drawInactiveMenuItem(bmp, 1, 5, "paste");
    else 
      drawMenuItem(bmp, 1, 5, "paste");
    
    drawMenuItem(bmp, 6, 5, "ins pdb");
    drawMenuItem(bmp, 11, 5, "add AAAA");
    return;
  }
  
  // menu for blocking residues
  if (mpDisplay->activeLeftRes == mpDisplay->activeRightRes) 
    if (  (mpDisplay->pProtein->resKey(mpDisplay->activeRightRes) == ACE)
       || (mpDisplay->pProtein->resKey(mpDisplay->activeRightRes) == NME) )
    {
      if (mClipboardProtein.isEmpty())
        drawInactiveMenuItem(bmp, 1, 5, "paste");
      else
        drawMenuItem(bmp, 1, 5, "paste");
      drawMenuItem(bmp, 6, 5, "delete");
      drawMenuItem(bmp, 11, 5, "ins pdb");
       
      return;
    }

  // menu for normal residues
  for (int n = 1; n <= 20; n++)
  {
    std::string s(1, getResChar(n));
    drawMenuItem(bmp, n, 1, s);
  }

  drawMenuItem(bmp, 21, 5, "copy");
  drawMenuItem(bmp, 26, 5, "cut");

  if (mClipboardProtein.isEmpty())
    drawInactiveMenuItem(bmp, 31, 5, "paste");
  else
    drawMenuItem(bmp, 31, 5, "paste");
  
  drawMenuItem(bmp, 36, 5, "delete");
  drawMenuItem(bmp, 41, 5, "ins pdb");
  drawMenuItem(bmp, 46, 5, "pdb out");

  // extra menu items for end residues
  if (mpDisplay->activeLeftRes == mpDisplay->activeRightRes) 
  {
    if (mpDisplay->activeRightRes == 0)
    {
      drawMenuItem(bmp, 51, 5, "block");
      if (mpDisplay->pProtein->isChargedNTerminal())
      {
        drawInactiveMenuItem(bmp, 56, 5, "charge");
        drawMenuItem(bmp, 61, 5, "neutral");
      }
      else
      {
        drawMenuItem(bmp, 56, 5, "charge");
        drawInactiveMenuItem(bmp, 61, 5, "neutral");
      }
    }
    if (mpDisplay->activeRightRes == mpDisplay->pProtein->nRes() - 1)
    {
      drawMenuItem(bmp, 51, 5, "block");
      if (mpDisplay->pProtein->isChargedCTerminal())
      {
        drawInactiveMenuItem(bmp, 56, 5, "charge");
        drawMenuItem(bmp, 61, 5, "neutral");
      }
      else
      {
        drawMenuItem(bmp, 56, 5, "charge");
        drawInactiveMenuItem(bmp, 61, 5, "neutral");
      }
    }
  }
}



void SequenceBox::draw(BITMAP *bmp)
{
  Protein& protein = *(mpDisplay->pProtein);
  text_mode(-1);
  int viewSize = mpDisplay->viewRightRes - mpDisplay->viewLeftRes + 1;
  
  // draw the sequence box
  for (int i = 0; i < viewSize; i+=1)
  {
    int rX = i * mW + mX;

    // draw the amino acid letters
    int actual_res = i + mpDisplay->viewLeftRes;
    if ( (actual_res >= mpDisplay->activeLeftRes) && (actual_res <= mpDisplay->activeRightRes) )
    {
      rectfill(bmp, rX, mY, rX + mW, mY + mH, GREEN);
      std::string s(1, getResChar(mpDisplay->pProtein->resKey(actual_res)));
      stringout(bmp, font, s, rX + 3, mY + 3, BLACK);
    }
    else
    {
      std::string s(1, getResChar(mpDisplay->pProtein->resKey(actual_res)));
      stringout(bmp, font, s, rX + 3, mY + 3, WHITE);
    }
    if ((i==mpDisplay->anchorRes1)||(i==mpDisplay->anchorRes2)||(i==mpDisplay->anchorRes3))
       rect(bmp, rX, mY, rX + mW, mY + mH, BLUE);
    else
       rect(bmp, rX, mY, rX + mW, mY + mH, WHITE);
   
    // draw secondary structure 
    if (      (protein.phi(actual_res) > -180) && (protein.phi(actual_res) <   0)
           && (protein.psi(actual_res) >   60) && (protein.psi(actual_res) < 180) )
      stringout_centre(bmp, font, "E", rX + 7, mY + 35, GREY);
    else if ( (protein.phi(actual_res) >  -90) && (protein.phi(actual_res) <  30)
           && (protein.psi(actual_res) >  -90) && (protein.psi(actual_res) <  30) )
      stringout_centre(bmp, font, "H", rX + 7, mY + 35, GREY);
    else if ( (protein.phi(actual_res) >    0) && (protein.phi(actual_res) <  90)
           && (protein.psi(actual_res) >    0) && (protein.psi(actual_res) <  90) )
      stringout_centre(bmp, font, "L", rX + 7, mY + 35, GREY);

    // draw numbered ticks
    int resNo = mpDisplay->pProtein->resNo(actual_res);
    if (0 == (resNo % 5))
      stringout_centre(bmp, font, double_to_string(resNo, 0, 0),
         rX + 7, mY + 24, GREY);
  }

  // draw end marker
  if (mpDisplay->viewRightRes == mpDisplay->pProtein->nRes() - 1)
  {
    int rX = (viewSize) * mW + mX;
    if ( (mIsMenu) && (mMenuRes == mpDisplay->pProtein->nRes()) )
    {
      rectfill(bmp, rX, mY, rX + mW, mY + mH, GREEN);
      rect(bmp, rX, mY, rX + mW, mY + mH, WHITE);
      stringout(bmp, font, ":", rX + 3, mY + 3, BLACK);
    }
    else
    {
      rect(bmp, rX, mY, rX + mW, mY + mH, WHITE);
      stringout(bmp, font, ":", rX + 3, mY + 3, WHITE);
    }
  }
  
  if (mIsMenu) 
    drawMenu(bmp);
}



int FocusBox::handle(int event)
{
  switch (event)
  {
    case WGT_MOUSE_B1_DOWN:
    case WGT_MOUSE_B1_DRAG:
    case WGT_MOUSE_B2_DOWN:
    case WGT_MOUSE_B2_DRAG:
    {
      int j = convertToSeqNo(mouse_x);
      if ( (j >= 0) && (j < mpDisplay->pProtein->nRes()) )
      {
        mpDisplay->focusRes = j;
        mpDisplay->focusAtom = CA;
        mpDisplay->resetFocus();
        if (j > mpDisplay->viewRightRes)
        {
          mpDisplay->viewRightRes = j;
          mpDisplay->viewLeftRes = mpDisplay->viewRightRes - SCROLLBAR_SIZE + 1;
        }

        if (j < mpDisplay->viewLeftRes)
        {
          mpDisplay->viewLeftRes = j;
          mpDisplay->viewRightRes = mpDisplay->viewLeftRes + SCROLLBAR_SIZE - 1;
        }
      }
     }

    return 1;
    break;
  }
  return 0;
}



void FocusBox::draw(BITMAP *bmp)
{
  if (mpDisplay->pProtein->isEmpty())
    return;
  text_mode(-1);
 
  int viewSize = mpDisplay->viewRightRes - mpDisplay->viewLeftRes + 1;
  for (int i = 0; i < viewSize; i+=1)
  {
    int rX = i*mW + mX;
    int actual_res = i + mpDisplay->viewLeftRes;
    if (actual_res == mpDisplay->focusRes)
      rectfill(bmp, rX, mY, rX + mW, mY + mH, PURPLE);
    if ((i==mpDisplay->anchorRes1)||(i==mpDisplay->anchorRes2)||(i==mpDisplay->anchorRes3))
       rect(bmp, rX, mY, rX + mW, mY + mH, BLUE);
    else
       rect(bmp, rX, mY, rX + mW, mY + mH, WHITE);
  }
}



bool ScrollBar::isInside(int x, int y)
{
  if (mpDisplay->pProtein->nRes() <= SCROLLBAR_SIZE) 
    return false;
  else
    return ( (x >= mX) && (x <(mX + mpDisplay->pProtein->nRes() * mW))
          && (y >= mY) && (y <(mY + mH)) );
}



int ScrollBar::handle(int event)
{
  switch (event)
  {
    case WGT_MOUSE_B1_DOWN:
    {
      int clickedRes = (int)((mouse_x - mX + 1) /((double)mW - 4.0) * mpDisplay->pProtein->nRes());
      if ( (clickedRes >= mpDisplay->viewLeftRes) && (clickedRes <= mpDisplay->viewRightRes) )
        mClickedResOffset = clickedRes - (mpDisplay->viewRightRes + mpDisplay->viewLeftRes) / 2;
      else
        mClickedResOffset = -1;
    }

    case WGT_MOUSE_B1_DRAG:
    {
      int newCenterRes = (int)(1.0 * (mouse_x - mX + 1) / (mW - 4) 
                                * mpDisplay->pProtein->nRes() - mClickedResOffset);
      int centerRes = (int)(1.0 * (mpDisplay->viewRightRes + mpDisplay->viewLeftRes) / 2);
      int disp = newCenterRes - centerRes;


      mpDisplay->viewLeftRes += disp;
      mpDisplay->viewRightRes += disp;

      if (mpDisplay->viewLeftRes < 0)
      {
        mpDisplay->viewLeftRes = 0;
        mpDisplay->viewRightRes = SCROLLBAR_SIZE - 1;
      }
      if (mpDisplay->viewRightRes > mpDisplay->pProtein->nRes() - 1)
      {
        mpDisplay->viewRightRes = mpDisplay->pProtein->nRes() - 1;
        mpDisplay->viewLeftRes = mpDisplay->viewRightRes - SCROLLBAR_SIZE + 1;
      }

      mpDisplay->resetFocus();
    }
    return 1;
    break;
  }
  return 0;
}



void ScrollBar::draw(BITMAP *bmp)
{
  if (mpDisplay->pProtein->isEmpty())
    return;
  if (mpDisplay->pProtein->nRes() <= SCROLLBAR_SIZE) return;

  int bar_size      = (int)(1.0 * (mpDisplay->viewRightRes - mpDisplay->viewLeftRes + 1) / mpDisplay->pProtein->nRes() * (mW-4));
  int baracceptorffset    = (int)(1.0 * (mpDisplay->viewLeftRes) / mpDisplay->pProtein->nRes() * (mW-4));
  int focusacceptorffset  = (int)(1.0 * (mpDisplay->focusRes) / mpDisplay->pProtein->nRes() * (mW-4));
  int focusacceptorffset2 = (int)(1.0 * (mpDisplay->focusRes + 1) / mpDisplay->pProtein->nRes() * (mW-4));
  int activeacceptorffset = (int)(1.0 * (mpDisplay->activeLeftRes) / mpDisplay->pProtein->nRes() * (mW-4));
  int activeacceptorffset2= (int)(1.0 * (mpDisplay->activeRightRes + 1) / mpDisplay->pProtein->nRes() * (mW-4));

  rect(bmp, mX + baracceptorffset + 1, mY + 1, mX + baracceptorffset + bar_size - 1, mY + mH - 2, BLACK);
  rectfill(bmp, mX + baracceptorffset + 2, mY + 2, mX + baracceptorffset + bar_size + 2, mY + mH - 2, WHITE);


  line(bmp, mX,         mY,        mX,         mY +mH - 1, WHITE);
  line(bmp, mX +mW - 1, mY,        mX +mW - 1, mY +mH - 1, WHITE);
  line(bmp, mX,         mY + mH/2, mX +mW - 1, mY + mH/2,  WHITE);

  rectfill(bmp, mX + activeacceptorffset + 2, mY + 3, mX + activeacceptorffset2 + 1, mY + mH - 5, GREEN);
  rectfill(bmp, mX + focusacceptorffset + 2,  mY + 5, mX + focusacceptorffset2 + 1,  mY + mH - 3, PURPLE);
}




/// Loop-mode control box



bool LoopBox::isInside(int x, int y)
{
  if (mpDisplay->pProtein->isEmpty()) return false;
  int actual_width = mpDisplay->mNLoopSolution*mW;
  if (actual_width == 0) return false;
  
  return((mouse_x >= mX) && (mouse_x <(mX + actual_width))
        && (mouse_y >= mY) && (mouse_y <= mY + mH) );
}



int LoopBox::handle(int event)
{
  switch (event)
  {
    case WGT_MOUSE_B1_DOWN:
    case WGT_MOUSE_B1_DRAG:
      {
        int i = (int)( (mouse_x - mX)/(double)mW );
        if ((i >= 0) && (i < mpDisplay->mNLoopSolution))
        {
          mpDisplay->changeLoopSolution(i);
          mpDisplay->resetFocusWithSameOrientation();
          mpDisplay->pProtein->extractBackboneAngles();
        }
      }
    break;
    default:
      return 0;
  }
  return 1;
}



void LoopBox::draw(BITMAP *bmp)
{
//  rectfill(bmp, mX, mY, mX + 16 * mW, mY + mH, GREY);
  if (mpDisplay->pProtein->isEmpty()) return;

  int nLoopSolution = mpDisplay->mNLoopSolution;
  
  if (nLoopSolution == 0) return;

  Protein& protein = *(mpDisplay->pProtein);

  // draw the loop-solution box
  for (int i = 0; i < nLoopSolution; i+=1)
  {
    int rX = i * mW + mX;

    if (i == mpDisplay->mILoopSolution)
    {
      rectfill(bmp, rX + 1, mY + 1, rX + mW - 1, mY + mH - 1, BLUE);
      rect(bmp, rX, mY, rX + mW, mY + mH, BLUE);
    }
    else
    {
      rect(bmp, rX, mY, rX + mW, mY + mH, BLUE);
    }
  }
}






