
//
//  ****************************************************************************
//
//  control.h
//
//  (C) Thu Mar 29 2004 Bosco K. Ho 
//
//  ****************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  ****************************************************************************
//

#ifndef _control_h_
#define _control_h_

#include "display.h"




class RamaBox : public Widget
{
  public:
    RamaBox(ProteinDisplay* box, int x, int y, int w, int h)
      : Widget(x, y, w, h), mpDisplay(box) {};
    int  handle(int event);
    void draw(BITMAP *b);
  private:
    ProteinDisplay* mpDisplay;
};



class ChiBox : public Widget
{
  public:
    ChiBox(ProteinDisplay* box, int x, int y, int w, int h)
      : Widget(x, y, w, h), mpDisplay(box)  {}
    int  handle(int event);
    void draw(BITMAP *b);
    virtual bool isInside(int x, int y);
    int iChi;
  private:
    ProteinDisplay* mpDisplay;
};



class SequenceBox : public Widget
{
  public:
    SequenceBox(ProteinDisplay* p, int x, int y, int w, int h)
      : Widget(x, y, w, h), mpDisplay(p), mIsMenu(false) {}

    bool     isInside(int x, int y);
    int      handle(int event);
    void     draw(BITMAP *b);

  protected:
    void     drawMenu(BITMAP *b);
    void     drawMenuBlock(BITMAP *bmp, int n, int width, std::string s, int foreColor, int backColor);
    void     drawMenuItem(BITMAP *bmp, int n, int width, std::string s);
    void     drawInactiveMenuItem(BITMAP *bmp, int n, int width, std::string s);
    int      convertToSeqNo(int x);
    void     checkBounds();
    int      getMenuIndex(int x, int y);
    void     deleteActiveRes();
    void     pasteClipboard();
    void     copyToClipboard();
    void     loadClipboardFromFile();

    int      mAnchorRes;
    int      mMenuRes;
    int      mIsMenu;
    Protein  mClipboardProtein;
    ProteinDisplay* mpDisplay;
};  



/// Sequence focus 
class FocusBox : public SequenceBox
{
  public:
    FocusBox(ProteinDisplay* box, int x, int y, int w, int h)
      : SequenceBox(box, x, y, w, h)  {}
    int handle(int event);
    void draw(BITMAP *b);
};



/// Scroll - Bar
class ScrollBar : public Widget
{
  public:
    ScrollBar(ProteinDisplay* box, int x, int y, int w, int h)
      : Widget(x, y, w, h), mpDisplay(box) {}
    int  handle(int event);
    void draw(BITMAP *b);
    bool isInside(int x, int y);
  private:
    int mClickedResOffset;
    ProteinDisplay* mpDisplay;
};



/// Loop-modes
class LoopBox : public Widget
{
  public:
    LoopBox(ProteinDisplay* box, int x, int y, int w, int h)
      : Widget(x, y, w, h), mpDisplay(box) {}
    int  handle(int event);
    void draw(BITMAP *b);
    bool isInside(int x, int y);
  private:
    ProteinDisplay* mpDisplay;
};

#endif

