//
//  **************************************************************************
//
//  polymer.h
//
//  (C) 2003 Bosco K. Ho 
//
//  Adapted from the The Biomolecule Toolkit (C) 2001, Eric Alm 
//  
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//

#ifndef polymer_h_
#define polymer_h_

#include <vector>
#include <iostream>

#include "molecule.h"
#include "atoms.h"
#include "residue.h"



const int MAX_RES_PARAM = 15;

// look-up table for blocks of atom iterators in a residue
struct Res 
{
  int            key;             
  std::string    name;
  int            no;
  double         param[MAX_RES_PARAM];
  AtomIterList   atomIterList;
  AtomIter       atomEnd;
};



class Polymer : public Molecule 
{
  public:
    Polymer(char chain_id = ' ', const std::string name = "untitled") :
            Molecule(chain_id, name), mResList() {};
    Polymer(const Polymer& rhs);
    ~Polymer() {}

    void         clear() { mResList.clear();  Molecule::clear(); };
    bool         isEmpty() { return nRes() == 0; };
            
    Atom&        atom(int i, int atomKey);
    bool         hasAtom(int i, int atomKey);
    void         insertAtom(int i, const Atom& a);
    void         insertAtom(int i, int atomKey);
    void         eraseAtom(int i, int atomKey);

    void         scanRes();
    int          nRes() const;

    AtomIter&    resAtomBegin(int i);
    AtomIter&    resAtomEnd(int i);
    int          nAtomInRes(int i)      { return mResList[i].atomIterList.size(); }

    int          resKey(int i)          { return mResList[i].key; }
    int          resNo(int i)           { return mResList[i].no; }
    std::string  resName(int i)         { return mResList[i].name; }

    void         insertRes(int i, const Res& r);
    void         eraseRes(int i);
    void         setAtomEnd(int i, AtomIter a); 
                 
    double       maxLength();
                 
    void         sproutAtom(int i, int a1, double length, double angle, double dihed,
                             int a2, int a3, int a4);
    std::string  resAtomName(int i, int a);
                 
    void         extractFragment(Polymer& fragment, int i, int j);
    void         insertFragment(const Polymer& fragment, int i);
                 
  protected:     
    std::vector<Res> mResList;
};


#endif

