/* -----------------------------------------------------------------------------
 * Tri-peptide loop closure
 *
 * Copyright (C) 2003 
 *       Chaok Seok, Evangelos Coutsias, Matthew Jacobson, and Ken Dill
 *       UCSF and Univeristy of New Mexico
 * Witten by Chaok Seok 2003.  
 *
 * files to be compiled with:
 *     tripeptide_closure.c
 *     sturm.h
 *     sturm.c
 *  
 * -----------------------------------------------------------------------------
 */

#ifndef _tripep_closure_h_
#define _tripep_closure_h_

#define MAX_LOOP_SOLN 16

void initialize_loop_closure(double b_len[], double b_ang[], double t_ang[]);
/* -----------------------------------------------------------------------------
 * connectivity of atoms:
 *    N1-A1-C1-...-N2-A2-C2-...-N3-A3-C3
 * 
 * input:
 *   - b_len[6]: bond lengths (A1-C1, C1-N2,..., N3-A3)
 *   - b_ang[7]: bond angles (N1-A1-C1, A1-C1-N2, ..., N3-A3-C3)
 *   - t_ang[2]: torsion angles (A1-C1-N2-A2, A2-C2-N3-A3)
 * 
 * angles in radians
 * 
 * -----------------------------------------------------------------------------
 */
 
void solve_3pep_poly(double r_n1[], double r_a1[], double r_a3[], 
                     double r_c3[], double r_soln_n[][3][3], 
	                   double r_soln_a[][3][3], double r_soln_c[][3][3], 
	                   int *n_soln);
/* -----------------------------------------------------------------------------
 * connectivity of atoms:
 *    N1-A1-C1-...-N2-A2-C2-...-N3-A3-C3
 * 
 * input: 
 *   - r_n1[3], r_a1[3], r_a3[3], r_c3[3]: 
 *        Cartesian coordinates of N and CA atoms of the first residue and
 *        CA and C atoms of the last (third) residue.
 * output:
 *   - n_soln: number of alternative loop closure solutions.
 *   - r_soln_n[max_soln][3][3], r_soln_a[max_soln][3][3], 
 *     r_soln_c[max_soln][3][3]: 
 *        Cartesian coordinates of loop closure solutions. 
 *          first dim: solution number
 *          second dim: residue number
 *          third dimension: x, y, z component
 *  
 * ----------------------------------------------------------------------
 */

#endif // _tripep_closure_h_
