//
//  ***************************************************************************
//
//  util.cpp
//  (C) 2004 Bosco K. Ho 
//
//  ***************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  ****************************************************************************
//

#include "util.h"
#include <iostream>     
#include <sstream>     
#include <iomanip>



std::string double_to_string(double val, int width, int precision)
{
  std::ostringstream ostr;
  if (width > 0)
    ostr << std::fixed << std::setw(width) << std::setprecision(precision) << val;     
  else
    ostr << std::fixed << std::setprecision(precision) << val;     
  return ostr.str();
}



void stringout(BITMAP *bmp, const FONT *f, const std::string& s, int x, int y, int color)
{
  textout(bmp, f, s.c_str(), x, y, color);
}



void stringout_centre(BITMAP *bmp, const FONT *f, const std::string& s, int x, int y, int color)
{
  textout_centre(bmp, f, s.c_str(), x, y, color);
}



void stringout_right(BITMAP *bmp, const FONT *f, const std::string& s, int x, int y, int color)
{
  textout_right(bmp, f, s.c_str(), x, y, color);
}



//******** drawing routines ********************



void wireframe (BITMAP *bmp, int vertices, int v[], int color) 
{
  for (int i=0; i<= (vertices-2); i+=1) 
    line (bmp, v[i*2], v[i*2 + 1], v[i*2 + 2], v[i*2 + 3], color);
  line (bmp, v[(vertices-1)*2], v[(vertices-1)*2 + 1], v[0], v[1], color);
}


// string to show next to mouse
static std::string mouseText = "";

void set_mouse_text(std::string newMouseText)
{
  mouseText = newMouseText;
}



void draw_mouse(BITMAP *buffer)
{
  draw_sprite(buffer, mouse_sprite, mouse_x, mouse_y);

  if (mouseText == "")
    return;

  stringout(buffer, font, mouseText, mouse_x + 12, mouse_y + 10,  BLACK);
  stringout(buffer, font, mouseText, mouse_x + 12, mouse_y + 11,  BLACK);
  stringout(buffer, font, mouseText, mouse_x + 13, mouse_y + 10,  BLACK);

  stringout(buffer, font, mouseText, mouse_x + 11, mouse_y + 9,   BLACK);
  stringout(buffer, font, mouseText, mouse_x + 11, mouse_y + 10,  BLACK);
  stringout(buffer, font, mouseText, mouse_x + 12, mouse_y + 9,   BLACK);

  stringout(buffer, font, mouseText, mouse_x + 14, mouse_y + 11,  BLACK);
  stringout(buffer, font, mouseText, mouse_x + 14, mouse_y + 12,  BLACK);
  stringout(buffer, font, mouseText, mouse_x + 13, mouse_y + 12,  BLACK);

  stringout(buffer, font, mouseText, mouse_x + 15, mouse_y + 12,  BLACK);
  stringout(buffer, font, mouseText, mouse_x + 15, mouse_y + 13,  BLACK);
  stringout(buffer, font, mouseText, mouse_x + 14, mouse_y + 13,  BLACK);

  stringout(buffer, font, mouseText, mouse_x + 13, mouse_y + 11,  WHITE);
}




