//
//  **************************************************************************
//
//  widget.cpp
//  (C) 2003 Bosco K. Ho 
//
//  Basic definition of a screen/mouse widget. Simple widgets are
//  included: clickable, toggle and state buttons. As well, a mouse drawing
//  function is included that displays text.
//
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//


#include "widget.h"

////********* screen widgets **************



int Button::handle (int event) 
{
  if (event == WGT_MOUSE_B1_DOWN) 
  {
    mIsMouseDown = true;
    return 1;
  }
  else if (event == WGT_MOUSE_B1_RELEASE) 
  {
    mIsMouseDown = false;
    if (isInside(mouse_x, mouse_y) && mpProc != NULL)
      (*mpProc)();
    return 1;
  }
  else
    return 0;
}



void Button::draw (BITMAP *bmp) 
{
  text_mode (-1);
  if ((mIsMouseDown) && isInside(mouse_x, mouse_y)) 
  {
    rectfill (bmp, mX, mY, mX + mW - 1, mY + mH - 1, WHITE);
    stringout_centre (bmp, font, mName, mX + mW/2, mY + 4, BLACK);
  }
  else 
  {
    rect (bmp, mX, mY, mX + mW - 1, mY + mH - 1, WHITE);
    stringout_centre (bmp, font, mName, mX + mW/2, mY + 4, WHITE);
  }
}



int ToggleButton::handle (int event) 
{
  if (event == WGT_MOUSE_B1_DOWN) 
  {
    *mpFlag = ! *mpFlag;
    if (mpProc != NULL)
      (*mpProc)();
    return 1;
  }
  else
    return 0;
}



void ToggleButton::draw (BITMAP *bmp) 
{
  text_mode (-1);
  if (*mpFlag) 
  {
    rectfill (bmp, mX, mY, mX + mW - 1, mY + mH - 1, WHITE);
    stringout_centre (bmp, font, mName, mX + mW/2, mY + 4, BLACK);
  }
  else 
  {
    rect (bmp, mX, mY, mX + mW - 1, mY + mH - 1, WHITE);
    stringout_centre (bmp, font, mName, mX + mW/2, mY + 4, WHITE);
  }
}



/// State selector


int StateSelector::handle (int event)
{
  if (event == WGT_MOUSE_B1_DOWN) 
  {
    *mpVal = mVal;
    if (mpProc != NULL)
      (*mpProc)();
    return 1;
  }
  return 0;
}


void StateSelector::draw (BITMAP *bmp) 
{
  text_mode (-1);
  if (*mpVal == mVal) 
  {
    rectfill (bmp, mX, mY, mX + mW - 1, mY + mH - 1, WHITE);
    stringout_centre (bmp, font, mName, mX + mW/2, mY + 4, BLACK);
  }
  else {
    rect (bmp, mX, mY, mX + mW - 1, mY + mH - 1, WHITE);
    stringout_centre (bmp, font, mName, mX + mW/2, mY + 4, WHITE);
  }
}



void WidgetList::draw(BITMAP *bmp)
{
  clear_bitmap(bmp);
  for (unsigned int i = 0; i < mWidgetList.size(); i++) 
    mWidgetList[i]->draw(bmp);
}



void WidgetList::handleMouse()
{
    // send input from mouse to mWidgetList
    if (mMouseB1Index != -1) 
    {
      if (mouse_b & 1) 
        mWidgetList[mMouseB1Index]->handle (WGT_MOUSE_B1_DRAG);
      else 
      {
        mWidgetList[mMouseB1Index]->handle (WGT_MOUSE_B1_RELEASE);
        mMouseB1Index = -1;
       }
    }
    else if (mouse_b & 1) 
    {
      for (int i = mWidgetList.size () - 1; i >= 0; i--) 
        if (mWidgetList[i]->isInside (mouse_x, mouse_y)) 
        {
          mMouseB1Index = i;
          mWidgetList[i]->handle (WGT_MOUSE_B1_DOWN);
          break;
        }
    }

    if (mMouseB2Index != -1) 
    {
      if (mouse_b & 2) 
        mWidgetList[mMouseB2Index]->handle (WGT_MOUSE_B2_DRAG);
      else 
      {
        mWidgetList[mMouseB2Index]->handle (WGT_MOUSE_B2_RELEASE);
        mMouseB2Index = -1;
       }
    }
    else if (mouse_b & 2) 
    {
      for (int i = mWidgetList.size () - 1; i >= 0; i--) 
        if (mWidgetList[i]->isInside (mouse_x, mouse_y)) 
        {
          mMouseB2Index = i;
          mWidgetList[i]->handle (WGT_MOUSE_B2_DOWN);
          break;
        }
    }

}


