//
//  **************************************************************************
//
//  widget.h 
//
//  (C) 2003 Bosco K. Ho 
//
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//

#ifndef _widget_h
#define _widget_h

#include <allegro.h>
#include <string>
#include <vector>
#include "util.h"




typedef enum 
{
  WGT_MOUSE_B1_DOWN, 
  WGT_MOUSE_B1_DRAG, 
  WGT_MOUSE_B1_RELEASE, 
  WGT_MOUSE_B2_DOWN, 
  WGT_MOUSE_B2_DRAG, 
  WGT_MOUSE_B2_RELEASE, 
};



///////////////////////////////////
/// a widget

class Widget 
{
public:
  Widget(int x, int y, int w, int h) :
    mX(x), mY(y), mW(w), mH(h) {}
  ~Widget() {}

  virtual int  handle (int event) { return 0; }
  virtual bool isInside (int x, int y)
    {  return ( (x >= mX) && (x < (mX + mW)) && (y >= mY) && (y < (mY + mH))); }
  virtual void draw (BITMAP *b)  {}

  int  mX, mY, mW, mH;
};



class WidgetList
{
public:
  WidgetList(): mMouseB1Index(-1), mMouseB2Index(-1) {}
  ~WidgetList() { clear(); }

  void clear()
  {
    for (unsigned int i = 0; i < mWidgetList.size(); i++) 
      delete mWidgetList[i];
    mWidgetList.clear();
  }

  void push_back(Widget* w) { mWidgetList.push_back(w); }
  void draw(BITMAP *bmp);
  void handleMouse();

  std::vector<Widget*> mWidgetList;
  int mMouseB1Index;
  int mMouseB2Index;
};



///////////////////////////////////
/// transparent button classes

typedef void CallBack (void);

class Button : public Widget
{
public:
  Button (int x, int y, int w, int h, std::string name, CallBack* proc) :
      Widget(x, y, w, h),  mName(name), mpProc(proc), mIsMouseDown(false) {}
  ~Button () {}

  virtual int  handle (int event);
  virtual void draw (BITMAP *b);
public:
  CallBack*   mpProc;
  std::string mName;
  bool        mIsMouseDown;
};




class ToggleButton : public Button 
{
public:
  ToggleButton (int x, int y, int w, int h, bool* flagPtr, std::string name, CallBack* proc) :
    Button(x, y, w, h, name, proc), mpFlag(flagPtr) {}
  ~ToggleButton () {}
  virtual int  handle (int event);
  virtual void draw (BITMAP *b);
public:
  bool*       mpFlag;
};



class StateSelector : public Button
{
public:
  StateSelector (int x, int y, int w, int h, int* valPtr, int val, std::string name, CallBack* proc) :
    Button (x, y, w, h, name, proc), mVal(val), mpVal(valPtr) {}
  int  handle (int event);
  void draw (BITMAP *b);
private:
  int* mpVal;
  int  mVal;
};


#endif
